"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chalk_1 = require("chalk");
var Table = require("cli-table");
var fs = require("fs-extra");
var path = require("path");
var os_1 = require("os");
var glob = require("glob");
var Mocha = require('mocha');
var util = require("../utility/utility");
var script;
function suites() {
    var config = util.getConfig();
    var placeholder = 'n/a';
    var table = new Table({
        head: ['Name', 'Number of tests']
    });
    if (config.suites) {
        config.suites.forEach(function (suite) {
            var number = suite.tests ? suite.tests.length : 0;
            table.push([
                suite.suiteName || placeholder,
                number
            ]);
        });
    }
    console.log(table.toString());
}
exports.suites = suites;
function runTest(testName, cmd) {
    var config = util.getConfig();
    if (!config.tests || config.tests.length == 0) {
        return console.error('No tests are configured.');
    }
    var profiles = cmd.userProfiles;
    //console.log(`Running test ${chalk.green(testName)} against ${config.testUrl}`);
    var userProfiles = profiles && profiles.length > 0 ? profiles : [""];
    var test = config.tests.find(function (x) { return x.testName == testName; });
    userProfiles.forEach(function (profile, index) {
        if (test) {
            var profileName = profile ? "using user profile " + chalk_1.default.green(profile) : '';
            _createPreScriptSnippet("Running test " + chalk_1.default.green(testName) + " against " + chalk_1.default.green(config.testUrl) + " " + profileName, config, profile);
            _generateTestScriptSnippet(test, config);
        }
        else {
            console.log('Could not find the test ' + testName + '.');
        }
        _createTestFile("test" + index);
    });
    _runTests();
}
exports.runTest = runTest;
function runSuite(suiteName, cmd) {
    var config = util.getConfig();
    var suites = [];
    var profiles = cmd.userProfiles;
    if (config.suites) {
        suites = config.suites.filter(function (x) { return x.suiteName == suiteName; });
    }
    if (!suites || suites.length == 0) {
        return console.error('No test suites are configured.');
    }
    //console.log(`Running suite ${chalk.green(suiteName)} against ${config.testUrl}`);
    var userProfiles = profiles && profiles.length > 0 ? profiles : [""];
    userProfiles.forEach(function (profile, index) {
        var profileName = profile ? "using user profile " + chalk_1.default.green(profile) : '';
        _createPreScriptSnippet("Running suite " + chalk_1.default.green(suiteName) + " against " + chalk_1.default.green(config.testUrl) + " " + profileName, config, profile);
        suites.forEach(function (suite) {
            if (suite.tests) {
                suite.tests.forEach(function (testName) {
                    var test = config.tests.find(function (x) { return x.testName == testName; });
                    if (test) {
                        _generateTestScriptSnippet(test, config);
                    }
                });
            }
        });
        //once the test scripts are generated
        _createTestFile("test" + index);
    });
    _runTests();
}
exports.runSuite = runSuite;
function _createPreScriptSnippet(description, config, profileID) {
    var puppeteerOpts = config.puppeteerOpts ? JSON.stringify(config.puppeteerOpts) : '{}';
    var pageObjects = _getAllPageObjectFiles(config);
    var userProfile = _getUserProfile(profileID, config);
    script = [
        "const {expect} = require('chai')",
        "const puppeteer = require('puppeteer')",
        os_1.EOL,
        "describe('" + description + "', function () {",
        "// Define global variables",
        "let browser",
        "let page",
        "let userProfile = " + userProfile,
        "" + pageObjects,
        "before(async function () {",
        "   browser = await puppeteer.launch(" + puppeteerOpts + ");",
        "   page = await browser.newPage();",
        "   await page.goto('" + config.testUrl + "');",
        '})',
        os_1.EOL,
        "after(async function () {",
        "   await page.close()",
        "   await browser.close()",
        "})",
        os_1.EOL,
    ].join(os_1.EOL);
}
function _getUserProfile(profileID, config) {
    var value = "{}";
    var profile = config.userProfiles.find(function (x) { return x.id == profileID; });
    if (profile) {
        value = JSON.stringify(profile.value);
    }
    return value;
}
function _getAllPageObjectFiles(config) {
    var returnValue = '';
    var fileContent = '';
    var files = glob.sync(config.rootTestPath + "/**/*.pageObject.js", {
        root: path.join(process.cwd(), config.rootTestPath)
    });
    if (files) {
        files.forEach(function (file) {
            fileContent = fs.readFileSync(file).toString();
            returnValue = [
                returnValue,
                fileContent
            ].join(os_1.EOL);
        });
    }
    return returnValue;
}
function _createPostScriptSnippet() {
    script = [
        script,
        '});'
    ].join(os_1.EOL);
}
function _createTestFile(fileName) {
    _createPostScriptSnippet();
    var file = path.join(process.cwd(), "runnableTest/" + fileName + ".js");
    fs.outputFileSync(file, script);
}
function _runTests() {
    var mocha = new Mocha({ timeout: 30000 });
    var files = glob.sync("/*.js", {
        root: path.join(process.cwd(), 'runnableTest')
    });
    if (files) {
        files.forEach(function (file) {
            mocha.addFile(file);
        });
        mocha.run(function (failures) {
            _cleanUpTests();
            process.exitCode = failures ? -1 : 0;
        });
    }
}
function _cleanUpTests() {
    var files = glob.sync("/*.js", {
        root: path.join(process.cwd(), 'runnableTest')
    });
    if (files) {
        files.forEach(function (file) {
            if (fs.existsSync(file)) {
                fs.removeSync(file);
            }
        });
    }
    var dir = path.join(process.cwd(), 'runnableTest');
    if (fs.ensureDirSync(dir)) {
        fs.rmdirSync(dir);
    }
}
function _generateTestScriptSnippet(test, config) {
    var rootPath = path.join(process.cwd(), config.rootTestPath);
    var filePath = path.join(rootPath, test.jsFilePath);
    if (test.jsFilePath == undefined || !fs.existsSync(filePath)) {
        return console.error("No file configured for test: " + chalk_1.default.cyanBright(test.testName) + " or the file does not exist");
    }
    var content = fs.readFileSync(filePath).toString();
    if (content) {
        _addJStoScript(content, test);
    }
    return;
}
function _addJStoScript(content, test) {
    var description = "#" + test.testName;
    if (test.testDescription) {
        description += " - " + test.testDescription;
    }
    script = [
        script,
        "describe('" + description + "', function() {",
        content,
        "});"
    ].join(os_1.EOL);
}
//# sourceMappingURL=run.js.map